
#include <iostream.h>
#include <String.h>

// Is the string a latitude or longitude in degrees-minutes-seconds form?
// Check for a string of the form DDDMMSSH where DDDMMSS are all digits 0-9
// and H is one of N,S,E,W or the lower case equivalents. If #value# fits
// those criteria, return true, otherwise return false.

static bool
really_dms(String value)
{
    static Regex dms("[0-9][0-9][0-9][0-5][0-9][0-5][0-9][nsewNSEW]", 1);

    return value.matches(dms);
}

// Given that the string #value# is of the form DDDMMSSH, convert it to
// fractional degrees representation (where MMSS is converted into a fraction
// of a degree). Set the sign so that Latitude North is positive, Latitude
// South is negitive, Longitude East is positive and Longitude West is
// negative.
//
// NB: Assume that the DDDMMSSH part is constant length (i.e., 0 min 0 sec is
// represented 0000) and that N,S,W,E, ... is always present.

static double
convert_dms(String value)
{
    String hemisphere = value.at(7, 1);
    String seconds = value.at(5, 2);
    String minutes = value.at(3, 2);
    String degrees = value.at(0, 3);

#if 0
    cout << "Hemi: " << hemisphere << endl
	<< "Sec: " << seconds << endl
	<< "Min: " << minutes << endl
	<< "Deg: " << degrees << endl;
#endif

    double secs = atoi((const char *)seconds) / 60.0;
    double mins = (atoi((const char *)minutes) + secs)/ 60.0;
    double val = (double)atoi((const char *)degrees) + mins;

    hemisphere.downcase();
    if (hemisphere == "s" || hemisphere == "w")
	val *= -1.0;
#if 0
    cout << "Val: " << val << " (mins : " << mins << " secs: " << secs
	 << ")" <<endl;
#endif

    return val;
}

int
main(int argc,  char *argv[])
{
    String dms = argv[1];

    if (really_dms(dms))
	cout << "DMS: " << convert_dms(dms) << endl;
    else
	cout << "Note DMS: " << dms << endl;
}
